const fs = require("fs");
const path = require("path");
const sass = require("sass");
const glob = require("glob");
const consola = require("consola");
const config = require("../config");
const targetGlob = `${config.paths.src}/components/**/*.scss`;
const packageConfig = require("../package.json");
const destFolder = `${config.paths.publishDest}/${packageConfig.version}/components`;
const concat = require("concat");
const globConcat = require("glob-concat");
const uglifycss = require("uglifycss");

function processComponents() {
  return new Promise((resolve, reject) => {
    glob(targetGlob, (err, matches) => {
      if (err) reject(err);
      matches.forEach((src) => {
        const fileName = path.basename(src, ".scss");
        const dest = path.resolve(destFolder, `./${fileName}.css`);
        fs.mkdirSync(destFolder, { recursive: true });
        const result = sass.compile(src, {
          loadPaths: [path.resolve(__dirname, "../node_modules")],
        });
        fs.writeFileSync(dest, result.css, "utf-8");
      });
      consola.success("Process completed");
      resolve();
    });
  });
}

function generateAllCSS(glob, dest) {
  return new Promise((resolve, reject) => {
    globConcat(glob, (err, matches) => {
      if (err) reject(err);
      concat(matches, dest);
      resolve();
    });
  });
}

function uglifyFiles(targetGlob) {
  return new Promise((resolve, reject) => {
    glob(targetGlob, (err, matches) => {
      if (err) reject(err);
      matches.forEach((src) => {
        const dest = src.replace(".css", ".min.css");
        const content = fs.readFileSync(src, "utf-8");
        const uglifiedContent = uglifycss.processString(content);
        fs.writeFileSync(dest, uglifiedContent, "utf-8");
      });
      resolve();
    });
  });
}

async function run() {
  const destConcat = path.resolve(destFolder, `./all.css`);
  const cssSrc = `${destFolder}/**/*.css`;
  await processComponents();
  await generateAllCSS(cssSrc, destConcat);
  await uglifyFiles(cssSrc);
}

run();
